"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var projectPackages = {};
var projectModules = {};
exports.excludeProjectModules = function (projectRoot, includeNodeModulesRegexp) { return function (modulePath) {
    var idx = modulePath.indexOf(path.sep + 'node_modules' + path.sep);
    if (idx >= 0) {
        if (includeNodeModulesRegexp && includeNodeModulesRegexp.test(modulePath)) {
            return false;
        }
        if (!projectModules[modulePath]) {
            var pkgPathStart = modulePath[idx + 14] !== '@' ? idx + 14 : modulePath.indexOf(path.sep, idx + 14) + 1;
            var pkgPathEnd = modulePath.indexOf(path.sep, pkgPathStart);
            if (pkgPathEnd < 0) {
                pkgPathEnd = modulePath.length;
            }
            var pkgPath = modulePath.substr(0, pkgPathEnd);
            if (!projectPackages[pkgPath]) {
                try {
                    projectPackages[pkgPath] =
                        fs.lstatSync(pkgPath).isSymbolicLink() && fs.realpathSync(pkgPath).indexOf(projectRoot) === 0;
                }
                catch (e) {
                    projectPackages[pkgPath] = false;
                }
            }
            projectModules[modulePath] = projectPackages[pkgPath];
        }
        return !projectModules[modulePath];
    }
    else {
        return false;
    }
}; };
var JSRuleFinder = (function () {
    function JSRuleFinder(builder) {
        this.builder = builder;
    }
    JSRuleFinder.prototype.findJSRule = function () {
        if (!this.jsRule) {
            var jsCandidates = ['\\.js$/', '\\.jsx?$/'];
            for (var _i = 0, _a = this.builder.config.module.rules; _i < _a.length; _i++) {
                var rule = _a[_i];
                for (var _b = 0, jsCandidates_1 = jsCandidates; _b < jsCandidates_1.length; _b++) {
                    var candidate = jsCandidates_1[_b];
                    if (String(rule.test).indexOf(candidate) >= 0) {
                        this.jsRule = rule;
                        break;
                    }
                }
            }
        }
        return this.jsRule;
    };
    JSRuleFinder.prototype.createJSRule = function () {
        if (this.jsRule) {
            throw new Error('js rule already exists!');
        }
        this.jsRule = { test: /\.js$/, exclude: exports.excludeProjectModules(this.builder.projectRoot) };
        this.builder.config.module.rules = this.builder.config.module.rules.concat(this.jsRule);
        return this.jsRule;
    };
    JSRuleFinder.prototype.findAndCreateJSRule = function () {
        return this.findJSRule() || this.createJSRule();
    };
    JSRuleFinder.prototype.findTSRule = function () {
        if (!this.tsRule) {
            var jsCandidates = ['\\.ts$/', '\\.tsx?$/'];
            for (var _i = 0, _a = this.builder.config.module.rules; _i < _a.length; _i++) {
                var rule = _a[_i];
                for (var _b = 0, jsCandidates_2 = jsCandidates; _b < jsCandidates_2.length; _b++) {
                    var candidate = jsCandidates_2[_b];
                    if (String(rule.test).indexOf(candidate) >= 0) {
                        this.tsRule = rule;
                        break;
                    }
                }
            }
        }
        return this.tsRule;
    };
    JSRuleFinder.prototype.createTSRule = function () {
        if (this.tsRule) {
            throw new Error('ts rule already exists!');
        }
        this.tsRule = { test: /\.ts$/, exclude: exports.excludeProjectModules(this.builder.projectRoot) };
        this.builder.config.module.rules = this.builder.config.module.rules.concat(this.tsRule);
        return this.tsRule;
    };
    JSRuleFinder.prototype.findAndCreateTSRule = function () {
        return this.findTSRule() || this.createTSRule();
    };
    Object.defineProperty(JSRuleFinder.prototype, "extensions", {
        get: function () {
            var result = [];
            this.findJSRule();
            this.findTSRule();
            var jsTestStr = String(this.jsRule ? this.jsRule.test : 'js');
            var tsTestStr = String(this.tsRule ? this.tsRule.test : '');
            if (tsTestStr.indexOf('tsx') >= 0) {
                result.push('tsx');
            }
            if (jsTestStr.indexOf('jsx') >= 0) {
                result.push('jsx');
            }
            if (tsTestStr.indexOf('ts') >= 0) {
                result.push('ts');
            }
            if (jsTestStr.indexOf('js') >= 0) {
                result.push('mjs');
                result.push('js');
            }
            return result;
        },
        enumerable: true,
        configurable: true
    });
    return JSRuleFinder;
}());
exports.default = JSRuleFinder;
//# sourceMappingURL=JSRuleFinder.js.map